import logging
from urllib.parse import urljoin

import pandas
import requests
from datetime import datetime


class ResultApi:
    def __init__(self, token, log_level="INFO"):
        """Python class to interrogate the Proofpoint result REST API.
        Each call to the API can be returned as a dict or a Pandas DataFrame

        Args:
            token (str): Proofpoint reporting API key
            log_level (str, optional): Log level. Defaults to "INFO".
        """
        self.token = token
        self.headers = {"Content-type": "application/json", "x-apikey-token": token}
        self.base_url = "https://results.us.securityeducation.com/api/reporting/v0.1.0/"
        self.now = datetime.utcnow()
        self.logger = logging.getLogger("default")
        self.logger.setLevel(logging.getLevelName(log_level))

        formatter = logging.Formatter(
            "%(asctime)s - %(funcName)s - %(levelname)s - %(message)s"
        )
        ch = logging.StreamHandler()

        ch.setFormatter(formatter)
        self.logger.addHandler(ch)

    def get_training(self, parameters={"page[size]": 10}, return_as_dataframe=False):
        """Returns information from training assignments.
        Contains functionality for a variety of filters. 
        Please see the "Parameter" section of this documentation for more information on these filters.
        https://proofpoint.securityeducation.com/api/reporting/documentation/#api-API_Endpoints-TrainingFunction

        Args:
            parameters (dict, optional): Parameters to tune the results from the API. Defaults to {"page[size]": 10}.
            return_as_dataframe (bool, optional): Set to True to return the response as a Pandas DataFrame. Defaults to False.

        Returns:            
                dict/pandas.DataFrame: Result as dict of DataFrame depending of return_as_dataframe arg.
        """
        url = urljoin(self.base_url, "training")
        response = requests.get(url, headers=self.headers, params=parameters)

        if response.status_code == 200:
            self.logger.debug(
                f"Successful response received for request on {response.request.url}"
            )
            if return_as_dataframe:
                self.logger.debug("Returning response as a DataFrame")
                df = pandas.json_normalize(response.json()["data"])
                df["UTCExtractDate"] = self.now
                return df
            else:
                self.logger.debug("Returning response as dict")
                return response.json()["data"]
        else:
            self.logger.error(
                f"Request returned an invalid status code: {response.status_code}"
            )
            raise ValueError(
                f"Request returned an invalid status code: {response.status_code}"
            )

    def get_cyberstrength(
        self, parameters={"page[size]": 10}, return_as_dataframe=False
    ):
        """Returns information from CyberStrength assessments. 
        Contains functionality for a variety of filters. 
        Please see the "Parameter" section of this documentation for more information on these filters.
        https://proofpoint.securityeducation.com/api/reporting/documentation/#api-API_Endpoints-CyberStrengthFunction

        Returns:
            dict/pandas.DataFrame: Result as dict of DataFrame depending of return_as_dataframe arg.
        """
        url = urljoin(self.base_url, "cyberstrength")
        response = requests.get(url, headers=self.headers, params=parameters)

        if response.status_code == 200:
            self.logger.debug(
                f"Successful response received for request on {response.request.url}"
            )
            if return_as_dataframe:
                self.logger.debug("Returning response as a DataFrame")
                df = pandas.json_normalize(response.json()["data"])
                df["UTCExtractDate"] = self.now
                return df
            else:
                self.logger.debug("Returning response as dict")
                return response.json()["data"]
        else:
            self.logger.error(
                f"Request returned an invalid status code: {response.status_code}"
            )
            raise ValueError(
                f"Request returned an invalid status code: {response.status_code}"
            )

    def get_phishing(self, parameters={"page[size]": 10}, return_as_dataframe=False):
        """Returns information for simulated phishing campaigns. 
        Contains functionality for a variety of filters. 
        Please see the "Parameter" section of this documentation for more information on these filters.
        https://proofpoint.securityeducation.com/api/reporting/documentation/#api-API_Endpoints-PhishingEventFunction

        Returns:
            dict/pandas.DataFrame: Result as dict of DataFrame depending of return_as_dataframe arg.
        """
        url = urljoin(self.base_url, "phishing")
        response = requests.get(url, headers=self.headers, params=parameters)

        if response.status_code == 200:
            self.logger.debug(
                f"Successful response received for request on {response.request.url}"
            )
            if return_as_dataframe:
                self.logger.debug("Returning response as a DataFrame")
                df = pandas.json_normalize(response.json()["data"])
                df["UTCExtractDate"] = self.now
                return df
            else:
                self.logger.debug("Returning response as dict")
                return response.json()["data"]
        else:
            self.logger.error(
                f"Request returned an invalid status code: {response.status_code}"
            )
            raise ValueError(
                f"Request returned an invalid status code: {response.status_code}"
            )

    def get_phishalarm(self, parameters={"page[size]": 10}, return_as_dataframe=False):
        """Returns PhishAlarm information. 
        Contains functionality for date filters, and enabling / disabling of users' tag information. 
        Please see the "Parameter" section of this documentation for more information on these filters.
        https://proofpoint.securityeducation.com/api/reporting/documentation/#api-API_Endpoints-PhishAlarmFunction

        Returns:
            dict/pandas.DataFrame: Result as dict of DataFrame depending of return_as_dataframe arg.
        """
        url = urljoin(self.base_url, "phishalarm")
        response = requests.get(url, headers=self.headers, params=parameters)

        if response.status_code == 200:
            self.logger.debug(
                f"Successful response received for request on {response.request.url}"
            )
            if return_as_dataframe:
                self.logger.debug("Returning response as a DataFrame")
                df = pandas.json_normalize(response.json()["data"])
                df["UTCExtractDate"] = self.now
                return df
            else:
                self.logger.debug("Returning response as dict")
                return response.json()["data"]
        else:
            self.logger.error(
                f"Request returned an invalid status code: {response.status_code}"
            )
            raise ValueError(
                f"Request returned an invalid status code: {response.status_code}"
            )

    def get_users(
        self,
        parameters={"page[size]": 10, "user_tag_enable": "TRUE"},
        return_as_dataframe=False,
    ):
        """Returns user information. 
        Contains functionality for an email filter, and enabling / disabling of users' tag information. 
        Please see the "Parameter" section of this documentation for more information on these filters.
        https://proofpoint.securityeducation.com/api/reporting/documentation/#api-API_Endpoints-UsersFunction

        Returns:
            dict/pandas.DataFrame: Result as dict of DataFrame depending of return_as_dataframe arg.
        """
        url = urljoin(self.base_url, "users")
        response = requests.get(url, headers=self.headers, params=parameters)

        if response.status_code == 200:
            self.logger.debug(
                f"Successful response received for request on {response.request.url}"
            )
            if return_as_dataframe:
                self.logger.debug("Returning response as a DataFrame")
                df = pandas.json_normalize(response.json()["data"])
                df["UTCExtractDate"] = self.now
                return df
            else:
                self.logger.debug("Returning response as dict")
                return response.json()["data"]
        else:
            self.logger.error(
                f"Request returned an invalid status code: {response.status_code}"
            )
            raise ValueError(
                f"Request returned an invalid status code: {response.status_code}"
            )
